package com.mortals.xhx.module.news.service.impl;
import com.mortals.xhx.module.news.model.*;
import com.mortals.xhx.module.news.service.NewsRecordService;
import com.mortals.xhx.module.news.service.NewsShareService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.news.dao.NewsDao;
import com.mortals.xhx.module.news.service.NewsService;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;

/**
* NewsService
* 新闻 service实现
*
* @author zxfei
* @date 2023-09-18
*/
@Service("newsService")
@Slf4j
public class NewsServiceImpl extends AbstractCRUDServiceImpl<NewsDao, NewsEntity, Long> implements NewsService {

    @Autowired
    private NewsShareService newsShareService;
    @Autowired
    private NewsRecordService newsRecordService;

    @Override
    public NewsEntity upNews(Long id) throws AppException {
        NewsEntity news = this.get(id);
        if(news==null){
            throw new AppException("新闻不存在");
        }
        NewsQuery update = new NewsQuery();
        update.setId(id);
        update.setUpNumsIncrement(1);
        return this.update(update);
    }

    @Override
    public NewsEntity viewNews(Long id, Context context) throws AppException {
        NewsEntity news = this.get(id);
        if(news==null){
            throw new AppException("新闻不存在");
        }
        if(context!=null && context.getUser()!=null) {
            NewsRecordEntity newsRecordEntity = newsRecordService.selectOne(new NewsRecordQuery().newsId(id).viewId(context.getUser().getId()));
            if(newsRecordEntity!=null){
                NewsRecordQuery updateRecord = new NewsRecordQuery();
                updateRecord.setId(newsRecordEntity.getId());
                updateRecord.setSumViewsIncrement(1);
                updateRecord.setViewTime(new Date());
                updateRecord.setUpdateTime(new Date());
                updateRecord.setUpdateUserId(context.getUser().getId());
                newsRecordService.update(updateRecord);
            }else {
                newsRecordEntity = new NewsRecordEntity();
                newsRecordEntity.initAttrValue();
                newsRecordEntity.setNewsId(id);
                newsRecordEntity.setViewId(context.getUser().getId());
                newsRecordEntity.setViewName(context.getUser().getRealName());
                newsRecordEntity.setCreateTime(new Date());
                newsRecordEntity.setCreateUserId(context.getUser().getId());
                newsRecordEntity.setSumViews(1);
                newsRecordService.save(newsRecordEntity);
            }
        }
        NewsQuery update = new NewsQuery();
        update.setId(id);
        update.setViewNumsIncrement(1);
        return this.update(update);
    }

    @Override
    public NewsEntity shareNews(Long id, Context context) throws AppException {
        NewsEntity news = this.get(id);
        if(news==null){
            throw new AppException("新闻不存在");
        }
        if(context!=null && context.getUser()!=null) {
            NewsShareEntity newsShareEntity = newsShareService.selectOne(new NewsShareQuery().newsId(id).createUserId(context.getUser().getId()));
            if(newsShareEntity!=null){
                NewsShareQuery updateShare = new NewsShareQuery();
                updateShare.setId(id);
                updateShare.setSumSharesIncrement(1);
                updateShare.setUpdateTime(new Date());
                updateShare.setUpdateUserId(context.getUser().getId());
                newsShareService.update(updateShare);
            }else {
                newsShareEntity = new NewsShareEntity();
                newsShareEntity.initAttrValue();
                newsShareEntity.setNewsId(id);
                newsShareEntity.setSumShares(1);
                newsShareEntity.setShareName(context.getUser().getRealName());
                newsShareEntity.setCreateTime(new Date());
                newsShareEntity.setCreateUserId(context.getUser().getId());
                newsShareService.save(newsShareEntity);
            }
        }
        NewsQuery update = new NewsQuery();
        update.setId(id);
        update.setShareNumsIncrement(1);
        return this.update(update);
    }
}